import { Vector3Utils } from "@minecraft/math";
import { system, world, GameMode, Player, ItemStack, EntityHealthComponent, InputPermissionCategory } from "@minecraft/server";
import { lifestealLogs } from "./databases";
import { lifesteal } from "./databases";
import { defaultSettings } from "./defaultSettings";
import { getScore, sendAllMessage, setScore } from "../stuff/commonUtils";
import { ghostSpectateMap, processSpectators } from "./commands/ghostsOnly/spectate";
import { handleRevive } from "./core_modules/reviveSystem";
import { processDeadPlayer } from "./core_modules/ghostHandling";

function addDefaultSettings() {
    for (const key in defaultSettings) {
        if (!lifesteal.has(key)) {
            lifesteal.set(key, defaultSettings[key as keyof typeof defaultSettings]);
        }
    }
}

addDefaultSettings();

system.runInterval(() => {
    if (system.currentTick % 200 === 0 && lifestealLogs.length > 500) lifestealLogs.length = 0;
    if (system.currentTick % 12000 === 0) addDefaultSettings();

    processSpectators();

    for (const player of world.getAllPlayers()) {
        if (system.currentTick % 1200 === 0) processDeadPlayer(player);

        handleRevive(player);

        const health = player.getComponent(`health`);
        const score = getScore(player, `ls-death`);
        setLoreItems(player);

        if (player.hasTag(`ls-dead`)) {
            if (lifesteal.get(`ghostbarrier`) !== `disabled`)
                handleGhostBarrier(player);

            if (!ghostSpectateMap.has(player.name)) {
                player.camera.clear();
                player.inputPermissions.setPermissionCategory(InputPermissionCategory.Movement, true);
            }
        }

        if (score > +lifesteal.get(`maxhearts`)) {
            exceededHearts(player);
        }

        if (player.hasTag(`heart-show_on`)) {
            player.onScreenDisplay.setActionBar(`§a${(health.currentValue / 2).toFixed(1)}/${score}`);
        }

        if (player.hasTag(`heart-display_on`) && system.currentTick % 20 === 0) {
            player.nameTag = `${player.name}\n§c${Math.round(health.currentValue / 2)}/${score}`;
        }

        handlePlayerHearts(player, score, health);
    }
}, 1);

function handlePlayerHearts(player: Player, score: number, health: EntityHealthComponent) {
    const effectiveMaxHealth = health.effectiveMax;

    if (!score && score !== 0) {
        const startAmount = lifesteal.get(`startamount`);
        setScore(player, `ls-death`, +startAmount);
        return;
    }

    if (score <= 0)
        player.addTag(`ls-dead`);

    if (score < 0)
        setScore(player, `ls-death`, 0);
    else if (score > 200)
        setScore(player, `ls-death`, 200);

    if (player.hasTag(`ls-dead`)) {
        player.setGameMode(GameMode.spectator);
        player.onScreenDisplay.setActionBar(`§cYou lost all your hearts, you are §ldead`);
        player.getComponent(`inventory`).container.clearAll();
    }

    if (lifesteal.get(`useplayerjson`) !== `true`) {
        handleCompatibilityMode(player, score, health);
        return;
    }

    if (score !== effectiveMaxHealth / 2) {
        if (score > 0) {
            player.triggerEvent(`lifesteal:health-${score}`);
            setScore(player, `hearts`, score);
        }

        system.run(() => {
            if (!player || !player.isValid) return;
            const currentHealth = health.currentValue;
            if (effectiveMaxHealth < health.effectiveMax && currentHealth === health.effectiveMax) {
                health.setCurrentValue(effectiveMaxHealth);
            }
        });
    }
}

function handleCompatibilityMode(player: Player, score: number, health: EntityHealthComponent) {
    player.triggerEvent(`lifesteal:health-10`);
    if (score <= 10) {
        if (health.currentValue / 2 > score) {
            health.setCurrentValue(score * 2);
        }
        return;
    }
    const level = score > 10 ? Math.floor((score - 11) / 2) + 1 : -1;

    player.addEffect(`health_boost`, 5, { amplifier: level, showParticles: false });

    if (health.currentValue / 2 > score) {
        health.setCurrentValue(score * 2);
    }
}


function handleGhostBarrier(player: Player) {
    const barrierDistance = +lifesteal.get(`ghostbarrier`);
    let spawnLocation = world.getDefaultSpawnLocation();
    if (spawnLocation.y === 32767) spawnLocation.y = 90;

    if (Vector3Utils.distance(player.location, spawnLocation) > barrierDistance) {
        const aLoc = Vector3Utils.subtract(player.location, spawnLocation);
        const teleportLocation = Vector3Utils.subtract(player.location, Vector3Utils.scale(Vector3Utils.normalize(aLoc), Vector3Utils.distance(player.location, spawnLocation) - barrierDistance + 0.5));
        player.teleport(teleportLocation);
    }
}

function exceededHearts(player: Player) {
    const current = getScore(player, `ls-death`);
    setScore(player, `ls-death`, +lifesteal.get(`maxhearts`));
    const amount = current - +lifesteal.get(`maxhearts`);
    const inventory = player.getComponent(`inventory`);
    const inventorySize = inventory.container.size;
    for (let i = 0; i < inventorySize; i++) {
        let item = inventory.container.getItem(i);
        if (!item || item.typeId !== `lifesteal:heart`) continue;
        inventory.container.setItem(i, new ItemStack(item.typeId, item.amount));
    }
    const sendCommandFeedback = world.gameRules.sendCommandFeedback;
    player.runCommand(`gamerule sendcommandfeedback false`);
    player.runCommand(`give @s lifesteal:heart ${amount}`);
    player.runCommand(`gamerule sendcommandfeedback ${sendCommandFeedback}`);
}

function setLoreItems(player: Player) {
    const inventory = player.getComponent(`inventory`).container;
    if (lifesteal.get(`antilore`) === `true` || inventory.emptySlotsCount === inventory.size || system.currentTick % 2 !== 0) return;

    for (let i = 0; i < inventory.size; i++) {
        const item = inventory.getSlot(i);
        if (!item.isValid || !item.hasItem()) continue;

        const lore = lores[item.typeId as keyof typeof lores];

        if (lore && item.getLore()[0] !== lore) {
            item.setLore(lore.split(`\n`));
        }
    }
}

const lores = {
    "lifesteal:beacon_of_life": "§r§7This is the beacon of life.\n§7Use this to revive a player.\n\n§7(( §o§fRight click/eat or do ?revive§7 to use ))",
    "lifesteal:heart_of_life": "§r§7This is the heart of life.\n§7Permanently increase your revive heart count\n\n§7(( §o§fRight click/eat to use ))",
    "lifesteal:heart_fragment": "§r§dUse 4 of these to craft a Heart.",
    "lifesteal:heart": "§r§7A heart is used to add a heart to your health.\n\n§7(( §o§fRight click/eat§7 to recieve a heart ))",
    "lifesteal:heart_essence": "§r§dUse this to craft a Heart.",
};