import { Player, system, world } from "@minecraft/server";
import { config } from "./config";
import { commands } from "./handler";


/**
 * Send a message to a player
 * @param player Player to send the message to
 * @param message Message the player will be sent
 * @param removeSpaces Whether to remove repeated spaces
 */
export function sendPlayerMessage(player: Player, message: string, removeSpaces: boolean = false) {
    const formattedMessage = removeSpaces ? message.replaceAll(/(\r\n|\r|\n)/g, `\n`).replaceAll(/  +/g, ``) : message;
    player.sendMessage(formattedMessage);
}

/**
 * Send every player a message
 * @param player Player that will send the message
 * @param message Message that will be sent
 * @param removeSpaces Whether to remove repeated spaces
 */
export function sendAllMessage(message: string, removeSpaces: boolean = false, requiredTag?: string) {
    const formattedMessage = removeSpaces ? message.replaceAll(/(\r\n|\r|\n)/g, `\n`).replaceAll(/  +/g, ``) : message;

    if (!requiredTag) {
        world.sendMessage(formattedMessage);
    } else {
        for (const player of world.getAllPlayers()) {
            if (player.hasTag(requiredTag)) {
                player.sendMessage(formattedMessage);
            }
        }
    }
}

export function helpMoreInfo(player: Player, args: string[]) {
    const cmdData = commands.find(c => c.name.toLowerCase() === args[0]?.toLowerCase() || c.aliases?.includes(args[0]?.toLowerCase()));
    if (!cmdData) return false;

    if (cmdData.staffOnly &&
        !cmdData.altHelpMenu &&
        !player.isOp() &&
        !player.hasTag(config.staffTag)
    ) {
        sendPlayerMessage(player, `§cYou do not have permission to do this!`);
        return true;
    }

    // Check if the command is staff only
    if ((cmdData.staffOnly || cmdData.altHelpMenu) && cmdData.helpMenu && cmdData.helpMenu !== `none`) {
        cmdData.helpMenu = cmdData.helpMenu.replaceAll(`§e`, `§9`).replaceAll(`§g`, `§u`);
    }

    // Check for an alternative help menu that is used when a player doesn't have ls-op
    if (cmdData.altHelpMenu &&
        !player.isOp() &&
        !player.hasTag(config.staffTag)) {
        sendPlayerMessage(player, `${cmdData.altHelpMenu}`);
        return true;
    }

    if (!cmdData.helpMenu) {
        sendPlayerMessage(player, `§cThere is no available help menu, this is a bug and should be reported to GamerDos. He probably forgot to put a help menu because hes stupid lmao (Discord: https://dsc.gg/gamerdos). Anyway heres a placeholder help menu:

Command Name: ${cmdData.name}
Staff Only: ${cmdData.staffOnly}
Command aliases: [
    ${cmdData.aliases?.join(`\n    `) ?? `none`}
]`);
        return true;
    }

    if (cmdData.helpMenu === `none`) {
        sendPlayerMessage(player, `§cNo help menu available for this command! (${config.prefix}${args[0]})`);
        return true;
    }

    sendPlayerMessage(player, `${cmdData.helpMenu}`);
    return true;
}

/**
 * Produces a random string
 * @param length Length of the random string
 * @param autoCharacters Enable premade characters for the string
 * @param characters If autoCharacters is false, use these characters instead of premade ones
 * @returns string
 */
export function random(length: number, autoCharacters: boolean, characters?: string) {
    if (autoCharacters) characters = `ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwyxz0123456789_`;
    let result = ``;
    for (let i = 0; i < length; i++) {
        result += characters.charAt(Math.floor(Math.random() * characters.length));
    }
    return result;
}

/**
 * Returns a random number between min and max
 * @param min Minimum number
 * @param max Maximum number
 */
export function randomN(min: number, max: number) {
    min = Math.ceil(min);
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min + 1)) + min;
}

/**
 * Gets a player's score
 * @param player Player to get the score from
 * @param objective Which objective to get the player's score from
 * @returns
 */
export function getScore(player: Player, objective: string) {
    try {
        return world.scoreboard.getObjective(objective)?.getScore(player.scoreboardIdentity);
    } catch { }
    return undefined;
}


/**
 * Sets a player's score
 * @param player Player to set the score from
 * @param objective Which objective to set the player's score from
 * @param score What score the player should be set to
 * @returns
 */
export function setScore(player: Player, objective: string, score: number) {
    return world.scoreboard.getObjective(objective).setScore(player, score);
}


/**
 * Adds to a player's score
 * @param player Player to add the score from
 * @param objective Which objective to add the player's score from
 * @param score How much score to add to the player
 * @returns
 */
export function addScore(player: Player, objective: string, amount: number) {
    return world.scoreboard.getObjective(objective).addScore(player.scoreboardIdentity, amount);
}

/**
 * stole this from papi and modified it (hi papi!!!)
 * @name getPlayer
 * @param {string} playerName The player name string to get from the world.
 * @example getPlayer("PAPISOP");
 * @remarks Gets the player requested from the world.
 * @returns {Player | null} Returns the player if the player with the playerName can be found, otherwise returns null.
 */
export const getPlayer = (playerName: string, id?: boolean, strict: boolean = true): Player | null => {
    const newPlayerName: string = playerName?.toLowerCase().trim().replace(/["'@]/g, "");

    const allPlayersArr: Player[] = world.getAllPlayers();
    const player = allPlayersArr.find(players =>
        players.name.toLowerCase() === newPlayerName ||
        !strict && players.name.toLowerCase().startsWith(newPlayerName) ||
        id && players.id === newPlayerName);

    return player ?? null;
};

/**
 * also stole this from papi (hi papi!!)
 * @name timeStamp
 * @param {number} time The milliseconds number which needs to be formatted.
 * @example timeStamp(20000);
 * @remarks Formats the milliseconds to a week, day, hour, minute, second format.
 * @returns {string} Returns the formatted timestamp string.
 */
export const timeStamp = (time: number): string => {
    let seconds: number = Math.floor(time / 1000);
    let minutes: number = Math.floor(seconds / 60);
    let hours: number = Math.floor(minutes / 60);
    let days: number = Math.floor(hours / 24);
    const weeks: number = Math.floor(days / 7);

    seconds %= 60;
    minutes %= 60;
    hours %= 24;
    days %= 7;

    const duration: string[] = [];
    if (weeks) duration.push(`${weeks} week${weeks > 1 ? `s` : ``}`);
    if (days) duration.push(`${days} day${days > 1 ? `s` : ``}`);
    if (hours) duration.push(`${hours} hour${hours > 1 ? `s` : ``}`);
    if (minutes) duration.push(`${minutes} minute${minutes > 1 ? `s` : ``}`);
    if (seconds) duration.push(`${seconds} second${seconds > 1 ? `s` : ``}`);

    if (duration.length) return duration.join(`, `);
    else return `0 seconds`;
};