import { ChatSendBeforeEvent, CustomCommand, CustomCommandOrigin, CustomCommandParamType, CustomCommandRegistry, CustomCommandResult, Player, system, world } from "@minecraft/server";
import { sendAllMessage, sendPlayerMessage } from "./commonUtils";
import { config } from "./config";

export type commandData = {
    /**
     * Name of the command
     */
    name: string;
    /**
     * Aliases for the command
     */
    aliases?: string[];
    /**
     * Whether its staff only or not
     */
    staffOnly?: boolean;
    /**
     * Help menu
    */
    helpMenu?: string;
    /**
     * Help menu to use when player isn't staff. If this is undefined, use helpMenu regardless
    */
    altHelpMenu?: string;
    /**
     * Callback
     */
    callback: (data: { player: Player, args: string[], cmdData: commandData; }) => void;
};

/**
 * Make a new command
 */
export class Command {
    static readonly commands: commandData[] = [];
    constructor(commandInfo: commandData) {
        if (!Command.commands.includes(commandInfo)) Command.commands.push(commandInfo);
    }
}

export const commands: commandData[] = Command.commands;

world.beforeEvents.chatSend.subscribe(data => {
    const findStart = config.prefixes.find(x => data.message.startsWith(x));
    if ((data.message.startsWith(config.prefix) && data.message !== config.prefix) || (findStart && data.message !== findStart)) {
        data.cancel = true;
        const args = data.message.slice(findStart ? findStart.length : config.prefix.length).match(/[^\s"']+|"[^"]+"|'[^']+'/g).map(x => x.replace(/"|'/g, ``));
        const command = args.shift();

        const cmdData = commands.find(c => c.name.toLowerCase() === command.toLowerCase() || c.aliases?.includes(command.toLowerCase()));

        if (!cmdData) {
            sendPlayerMessage(data.sender, `§cUnknown command: "${command}". Do ${config.prefix}help for a list of valid commands.`);
            return;
        }
        if (cmdData.staffOnly && !data.sender.hasTag(config.staffTag) && !data.sender.isOp()) return sendPlayerMessage(data.sender, `§cYou do not have permission to run this command!`);

        const player = data.sender;

        if (!cmdData.helpMenu) cmdData.helpMenu = `§cThere is no available help menu, this is a bug and should be reported to GamerDos. He probably forgot to put a help menu because hes stupid lmao (Discord: https://dsc.gg/gamerdos). Placeholder help menu:

Command Name: ${cmdData.name}
Staff Only: ${cmdData.staffOnly}
Command aliases: [
    ${cmdData.aliases?.join(`\n    `) ?? `none`}
]`;

        system.run(() => {
            cmdData.callback({ player, args, cmdData: cmdData });
        });
    }
});

type SlashCommandInfo = {
    commandInfo: CustomCommand,
    callback: (origin: CustomCommandOrigin, ...args: any[]) => CustomCommandResult | undefined;
};

type CommandParamEnumInfo = {
    name: string,
    values: string[];
};

export class SlashCommandBuilder {
    static readonly commands: SlashCommandInfo[] = [];
    static readonly enums: CommandParamEnumInfo[] = [];

    static createCommand(commandData: SlashCommandInfo) {
        SlashCommandBuilder.commands.push(commandData);
    }

    static createEnum(name: string, values: string[]) {
        SlashCommandBuilder.enums.push({
            name,
            values
        });
    }

    static registerCommands(customCommandRegistry: CustomCommandRegistry) {
        this.registerEnums(customCommandRegistry);

        for (const slashCommand of SlashCommandBuilder.commands) {
            customCommandRegistry.registerCommand(slashCommand.commandInfo, slashCommand.callback);
        }
    }

    private static registerEnums(customCommandRegistry: CustomCommandRegistry) {
        for (const customEnum of SlashCommandBuilder.enums) {
            customCommandRegistry.registerEnum(customEnum.name, customEnum.values);
        }
    }
}